@@ -1,5 +1,4 @@
 from collections import OrderedDict
-from collections import defaultdict
 import json
 import logging
 import operator
@@ -24,6 +23,7 @@ from .trace import bfs_order
 from .trace import child_map
 from .trace import copy_trace
 from .trace import root_span
+from .trace import trace_id as get_trace_id
 
 
 log = logging.getLogger(__name__)
@@ -116,25 +116,36 @@ def _normalize_traces(traces: List[Trace]) -> List[Trace]:
 
 
 def _match_traces(t1s: List[Trace], t2s: List[Trace]) -> List[Tuple[Trace, Trace]]:
-    similarities: Dict[TraceId, List[Tuple[TraceId, int]]] = defaultdict(lambda: [])
-    t1_map: Dict[TraceId, Trace] = {}
-    t2_map: Dict[TraceId, Trace] = {}
-    for t1 in t1s:
-        t1_trace_id = t1[0]["trace_id"]
-        t1_map[t1_trace_id] = t1
-        for t2 in t2s:
-            t2_trace_id = t2[0]["trace_id"]
-            t2_map[t2_trace_id] = t2
-            similarities[t1_trace_id].append((t2_trace_id, _trace_similarity(t1, t2)))
+    t1_map: Dict[TraceId, Trace] = {get_trace_id(t): t for t in t1s}
+    t2_map: Dict[TraceId, Trace] = {get_trace_id(t): t for t in t2s}
+    # Note: cannot assume trace_ids are unique between t1s and t2s (since snapshot normalizes them)
+    similarities: List[Tuple[TraceId, TraceId, int]] = []
 
+    # Map the similarity of every trace in t1s to every trace in t2s
+    for t1_trace_id, t1 in t1_map.items():
+        for t2_trace_id, t2 in t2_map.items():
+            similarities.append((t1_trace_id, t2_trace_id, _trace_similarity(t1, t2)))
+
+    # Sort the similarities by score
+    similarities = sorted(similarities, key=lambda x: x[2], reverse=True)
+
+    # Go over the scores and match the traces
+    matched_t1s = set()
+    matched_t2s = set()
     matches: List[Tuple[Trace, Trace]] = []
-    tids_to_match = set(similarities.keys())
-    while tids_to_match:
-        tid = tids_to_match.pop()
-        match_tid, match_score = max(similarities[tid], key=lambda t: t[1])
-        matches.append((t1_map[tid], t2_map[match_tid]))
+    for (t1_trace_id, t2_trace_id, score) in similarities:
+        if (t1_trace_id in matched_t1s) or (t2_trace_id in matched_t2s):
+            continue
+        matches.append((t1_map[t1_trace_id], t2_map[t2_trace_id]))
+        matched_t1s.add(t1_trace_id)
+        matched_t2s.add(t2_trace_id)
 
-    assert tids_to_match == set(), f"Unmatched traces {tids_to_match}"
+    assert len(matched_t1s) == len(
+        t1_map
+    ), f"Unmatched expected traces {set(t1_map.keys()) - matched_t1s}"
+    assert len(matched_t2s) == len(
+        t2_map
+    ), f"Unmatched received traces {set(t2_map.keys()) - matched_t2s}"
     return matches
 
 
@@ -253,6 +264,35 @@ def snapshot(
             _compare_traces(exp, rec, set(ignored))
 
 
+def _ordered_span(s: Span) -> OrderedDictType[str, TopLevelSpanValue]:
+    """Order the span to be more human readable."""
+    d = OrderedDict()
+    order = [
+        "name",
+        "service",
+        "resource",
+        "trace_id",
+        "span_id",
+        "parent_id",
+        "type",
+        "error",
+        "meta",
+        "metrics",
+    ]
+    for k in order:
+        if k in s:
+            if k in ["meta", "metrics"]:
+                # Sort the meta and metrics dictionaries alphanumerically
+                d[k] = OrderedDict(sorted(s[k].items(), key=operator.itemgetter(0)))  # type: ignore
+            else:
+                d[k] = s[k]  # type: ignore
+
+    # Add the rest of the attributes in alphanumeric order.
+    for k in sorted(set(s.keys()) - set(order)):
+        d[k] = s[k]  # type: ignore
+    return d  # type: ignore
+
+
 def _snapshot_trace_str(trace: Trace) -> str:
     cmap = child_map(trace)
     stack: List[Tuple[int, Span]] = [(0, root_span(trace))]
@@ -265,33 +305,8 @@ def _snapshot_trace_str(trace: Trace) -> str:
             else:
                 stack.insert(0, (prefix + 3, child))
 
-        def ordered_span(s: Span) -> OrderedDictType[str, str]:
-            """Order the span to be more human readable."""
-            d = OrderedDict()
-            for k in [
-                "name",
-                "service",
-                "resource",
-                "trace_id",
-                "span_id",
-                "parent_id",
-                "error",
-            ]:
-                if k in s:
-                    d[k] = s[k]  # type: ignore
-            for k in sorted(set(s.keys()) - {"name", "service", "resource"}):
-                d[k] = s[k]  # type: ignore
-
-            d["meta"] = OrderedDict(
-                sorted(s["meta"].items(), key=operator.itemgetter(0))
-            )
-            d["metrics"] = OrderedDict(
-                sorted(s["metrics"].items(), key=operator.itemgetter(0))
-            )
-            return d
-
         s += textwrap.indent(
-            json.dumps(ordered_span(span), indent=2), " " * (prefix + 2)
+            json.dumps(_ordered_span(span), indent=2), " " * (prefix + 2)
         )
         if stack:
             s += ",\n"
